
### code is obtain from Henderson et al. (2019) ###

gmfr.data <- function(x, y) {
  # browser()
  n<-length(x)
  ssd<-sum((x-y)^2) # Sum of square difference;
  msd<-ssd/n  # Mean square difference;
  xmean<-sum(x)/n
  ymean<-sum(y)/n # Mean values of x and y;
  s_xx<-sum((x-xmean)^2)
  s_yy<-sum((y-ymean)^2)
  s_xy<-sum((x-xmean)*(y-ymean))
  s_xyx<-sum(((x-xmean)+(y-xmean))^2)
  d<-1-(ssd/s_xyx) # Calculating Willmott's Index of Agreement
  rsq<-(s_xy)^2/(s_xx*s_yy) # R-square;
  spod<-sum((abs(xmean-ymean)+abs(x-xmean))*(abs(xmean-ymean)+abs(y-ymean))) # Sum of potential difference;
  ac<-1-ssd/spod  # Agreement coefficient;
  b_yvsx<-sqrt(s_yy/s_xx) # Estimate of b for GMFR regression y=a+bx;
  a_yvsx<-ymean-b_yvsx*xmean # Estimate of a for GMFR regression y=a+bx;
  b_xvsy<-sqrt(s_xx/s_yy) # Estimate of b for GMFR regression x=a+by;
  a_xvsy<-xmean-b_xvsy*ymean # Estimate of a for GMFR regression x=a+by;
  
  yhat<-a_yvsx+b_yvsx*x # Prediction of y;
  xhat<-a_xvsy+b_xvsy*y          # Prediction of x;
  spd_uns<-sum(abs(x-xhat)*abs(y-yhat))     # Unsystematic sum of product-difference;
  spd_sys<-ssd-spd_uns                   # Systematic sum of product-difference;
  ac_uns<-1-spd_uns/spod                     # Unsystematic agreement coefficient;
  ac_sys<-1-spd_sys/spod                     # Systematic agreement coefficient;
  mpd_uns<-spd_uns/n # Unsystematic mean product-difference;
  mpd_sys<-spd_sys/n # Systematic mean product-difference;
  rmsd<-sqrt(msd)  # Root mean of square difference;
  rmpd_uns<-sqrt(mpd_uns) # Unsystematic square root of mean product-difference;
  rmpd_sys<-sqrt(mpd_sys) # Systematic square root of mean product-difference;
  mse_sys<-(sum((x-yhat)^2))/n # Calculating Willmott's measures of agreement (systematic and unsystematic )
  mse_uns<-(sum((y-yhat)^2))/n
  mse<-mse_sys + mse_uns
  prop_uns<-mpd_uns/msd 
  prop_sys<-mpd_sys/msd 
  
  ##############################
  data <- data.frame(x, y, yhat)
  
  p <- ggplot(data) +
    geom_point(aes(x, y), size = 1) +
    geom_abline(slope = 1, intercept = 0, col = "red") +
    geom_line(aes(x = x, y = yhat), col = 'blue') +
    coord_fixed()
  
  return(list(p, rmsd, ac_uns, ac_sys, ac))
  
}
